@@ -1,9 +1,9 @@
 package Class::Container;
 
-$VERSION = '0.09';
+$VERSION = '0.11';
 $VERSION = eval $VERSION if $VERSION =~ /_/;
 
-my $HAVE_WEAKEN = 0;
+my $HAVE_WEAKEN;
 BEGIN {
   eval {
     require Scalar::Util;
@@ -127,21 +127,30 @@ sub all_specs
 
 sub dump_parameters {
   my $self = shift;
-  my $class = ref $self;
-  die "dump_parameters() can only be called as an object method, not a class method" unless $class;
+  my $class = ref($self) || $self;
   
   my %params;
   foreach my $param (keys %{ $class->validation_spec }) {
     next if $param eq 'container';
     my $spec = $class->validation_spec->{$param};
-    next if (exists $spec->{default} || $spec->{optional}) and !defined $self->{$param};
-    $params{$param} = $self->{$param};
+    if (ref($self) and defined $self->{$param}) {
+      $params{$param} = $self->{$param};
+    } else {
+      $params{$param} = $spec->{default} if exists $spec->{default};
+    }
   }
   
   foreach my $name (keys %{ $class->get_contained_object_spec }) {
-    my $subparams = ($self->{container}{contained}{$name}{delayed} ?
-		     $self->{container}{contained}{$name}{args} :
-		     $params{$name}->dump_parameters);
+    next unless ref($self);
+    my $contained = ($self->{container}{contained}{$name}{delayed} ?
+		     $self->delayed_object_class($name) :
+		     $params{$name});
+    
+    my $subparams = UNIVERSAL::isa($contained, __PACKAGE__) ? $contained->dump_parameters : {};
+    
+    my $more = $self->{container}{contained}{$name}{args} || {};
+    $subparams->{$_} = $more->{$_} foreach keys %$more;
+    
     @params{ keys %$subparams } = values %$subparams;
     delete $params{$name};
   }
